 #! /bin/sh
#written by guoliang@nfschina.com

data_dir="/etc/keyfiles/datas"
lst_file="/etc/keyfiles/.keyfiles.lst"
ss_lst_file="/etc/keyfiles/datas/sha1sums.lst"
log_file="/etc/keyfiles/kfc.log"

function report_error() {
 echo `date -R` KFC Error:$1 >> $log_file
}

function report_info() {
 echo `date -R` KFC Info:$1 >> $log_file
}

function report_prompt() {
 echo `date -R` KFC Prompt:$1 >> $log_file
}


function report_warning() {
 echo `date -R` KFC Warning:$1 >> $log_file
}

#pre-checking
#$1 is action, $2 is param num by cmdline
function pre_check() {
 
 if [ $1 != start -a $1 != stop -a $1 != restore -a $1 != verify -a $1 != recover -a $1 != check -a $1 != add -a $1 != del ]; then
  report_error "Please input action start/stop/restore/verify/recover file/check file/add file/del file"
  return 1
 fi

 if [ $1 == check -o $1 == recover -o $1 == add -o $1 == del ]; then
  if [ $2 != 2 ]; then
   report_error "$1 action need 1 params. Please input action start/stop/restore/verify/recover file/check file/add file/del file"
   return 1
  fi
 else
  if [ $2 != 1 ]; then
   report_error "$1 action need 0 params. Please input action start/stop/restore/verify/recover file/check file"
   return 1
  fi
 fi
 
 if [ -d $data_dir -a -f $ss_lst_file ]; then
  if [ $1 == start ]; then
   report_warning "The data dir and(or) sha1sum file exists, shouldn't start again. you may stop it first or manually rmdir $data_dir"
   #return 1
  fi
 else
  if [ $1 != start ]; then
   report_error "The data dir or sha1sum file not exists, shouldn't do action $1"
   return 1
  fi
 fi

 if [ `id -u` -ne 0 ]; then
  report_error "The script should be runned by root!"
  return 1
 fi

 if [ ! -f $lst_file ]; then
  report_error "lst file $lst_file is not keyfile!"
  return 1
 fi

 if [ `cat $lst_file |wc -l` -ne `sed /^$/d $lst_file | uniq | wc -l` ]; then
  report_error "lst file $lst_file has empty line or duplicate line"
  return 1
 fi

 if [ ! -s $lst_file ]; then
  report_error "the length of lst file $lst_file is 0 "
  return 1
 fi

 return 0
}

function is_keyfile() {
 local found=0
 for line in `cat $lst_file`
 do
  if [ $line == $1 ]; then
   found=1
   break
  fi
 done
 if [ $found -ne 1 ]; then
  report_error "File $1 is not keyfile"
  return 1 
 fi
}

function recover_file() {
 ss_fname=`echo "$1" | sha1sum | cut -d ' ' -f 1`
 cp $data_dir/$ss_fname $1
 cmd_result=$?
 if [ $cmd_result -ne 0 ]; then
  report_error "Execute command cp $data_dir/$ss_fname $2 with result $cmd_result"
  exit -1
 else
  report_info "recover file $1 Success"
 fi
}

function check_file() {
 ss_fname=`echo "$1" | sha1sum | cut -d ' ' -f 1`
 ss_file=`sha1sum $1 | cut -d ' ' -f 1`
 ss_orig=`grep "$ss_fname:" $ss_lst_file | cut -d ':' -f 2`
 if [ $ss_orig != $ss_file ]; then
  report_error "Check file $1 error with new sha1sum is $ss_file and old sha1sum is $ss_orig"
  return 1
 else
  report_info "Check file $1 Success!"
  return 0
 fi
}


pre_check $1 $#

if [ $? -ne 0 ]; then
 exit -1
fi

case "$1" in
 start)
  report_info "Start keyfiles checking by backup all files!"
  rm -rf $data_dir
  mkdir $data_dir
  ss_str=""
  newline="\n"
  for line in `cat $lst_file`
  do
   if [ -f "$line" ]; then
    ss_fname=`echo "$line" | sha1sum | cut -d ' ' -f 1`
    ss_file=`sha1sum "$line" | cut -d ' ' -f 1`
    line_sha1sum=$ss_fname:$ss_file$newline
    #doesn't consider the attr of file now   
    cp $line $data_dir/$ss_fname
    ss_str=$ss_str$line_sha1sum
   else
    report_error "File '$line' doesnot exist! rmdir $data_dir and exit"
    rm -rf $data_dir
    exit -1
   fi
  done
  echo -e $ss_str > $ss_lst_file
  report_info "Start Success!"  
  ;;

 stop)
  report_prompt "Stop keyfiles checking!"
  rm -rf $data_dir
  report_info "Stop Success!"
  ;;
 
 restore)
  report_prompt "Restore all files"
  for line in `cat $lst_file`
  do
   recover_file $line 
  done
  report_info "Restore Success!"
  ;;

 recover)
  if [ $# -ne 2 ]; then
   report_error "kfc.sh recover should give recover Filename"
   exit -1
  fi
  is_keyfile $2
  recover_file $2
  report_info "Recover Success!"
  ;;

 check)
  if [ $# -ne 2 ]; then
   report_error "kfc.sh check should give check Filename"
   exit -1
  fi
  is_keyfile $2
  check_file $2
  exit $?
  ;;

 verify)
  report_prompt "start verify all files"
  has_error=0
  for line in `cat $lst_file`
  do
   check_file $line
   if [ $? -ne 0 ]; then
    has_error=1
   fi
  done
  if [ $has_error -eq 0 ]; then
   report_info "Verify Success!"
  fi
  ;;

 add)
  report_prompt "add file"
  is_keyfile $2
  if [ $? -eq 0 ]; then
   report_error "File $2 is already keyfile"
   exit -1
  fi
  if [ ! -f $2 ]; then
   report_error "File $2 not exists"
   exit -1
  fi
  echo $2 >> $lst_file
  
  ss_fname=`echo "$2" | sha1sum | cut -d ' ' -f 1`
  ss_file=`sha1sum "$2" | cut -d ' ' -f 1`
  line_sha1sum=$ss_fname:$ss_file$newline
  #doesn't consider the attr of file now   
  cp $2 $data_dir/$ss_fname
  echo line_sha1sum >> $ss_lst_file
  report_info "Add $2 Success!"
  ;;

 del)
  report_prompt "del file"
  is_keyfile $2
  if [ $? -ne 0 ]; then
   report_error "File $2 is not keyfile"
   exit -1
  fi
  
  line_no=`grep -x -n "$2" $lst_file | cut -d ":" -f 1`
  sed -i "$line_no d" $lst_file
  
  ss_fname=`echo "$2" | sha1sum | cut -d ' ' -f 1`
  line_no=`grep -n "$ss_fname:" $ss_lst_file | cut -d ":" -f 1`
  sed -i "$line_no d" $ss_lst_file

  rm $data_dir/$ss_fname
  report_info "Del $2 Success!"
  ;;

*)
  exit -1;
esac

exit 0
